from PyQt5.QtDBus import QDBusArgument, QDBusVariant, QDBusConnection, QDBusMessage, QDBusPendingCallWatcher, QDBusPendingReply
from PyQt5.QtCore import QByteArray, QObject, pyqtSlot
from PyQt5.QtWidgets import QWidget, QApplication
from typing import Dict, Any


class DynamicLauncherPortal(QObject):
    def __init__(self) -> None:
        super().__init__()
        self._connection = QDBusConnection.sessionBus()

    @pyqtSlot(QDBusMessage)
    def _install_launcher(self, msg: QDBusMessage) -> None:
        if msg.arguments()[0] != 0:
            return

        install_message = QDBusMessage.createMethodCall("org.freedesktop.portal.Desktop", "/org/freedesktop/portal/desktop", "org.freedesktop.portal.DynamicLauncher", "Install")
        install_message.setArguments([msg.arguments()[1]["token"], self._desktop_id, self._desktop_file, {}])
        result = self._connection.call(install_message)
        print(result.errorMessage())

    def create_launcher(self, name: str, icon: bytes, options: Dict[str, Any], desktop_id: str, desktop_file: str) -> bool:
        self._desktop_id = desktop_id
        self._desktop_file = desktop_file

        arg = QDBusArgument()
        arg.beginStructure()
        arg.add("bytes")
        arg.add(QDBusVariant(QByteArray(icon)))
        arg.endStructure()

        message = QDBusMessage.createMethodCall("org.freedesktop.portal.Desktop", "/org/freedesktop/portal/desktop", "org.freedesktop.portal.DynamicLauncher", "PrepareInstall")
        message.setArguments(["", name, QDBusVariant(arg), options])
        result = self._connection.call(message)

        if result.errorName() == "org.freedesktop.DBus.Error.UnknownMethod":
            return False

        self._connection.connect("org.freedesktop.portal.Desktop", result.arguments()[0], "org.freedesktop.portal.Request", "Response", self._install_launcher)

        return True


def main():
    app = QApplication([])

    w = QWidget()
    w.show()

    dy = DynamicLauncherPortal()
    dy.create_launcher("Portaltest", b"test", {}, "org.kde.konsole.desktop", "")

    app.exec()


main()
